
FruitPreparerExtended = {}
FruitPreparerExtended.modDir = g_currentModDirectory
FruitPreparerExtended.modName = g_currentModName

function FruitPreparerExtended.initSpecialization()

	g_workAreaTypeManager:addWorkAreaType("fruitPreparer", false)

end



function FruitPreparerExtended.prerequisitesPresent(specializations)

	return SpecializationUtil.hasSpecialization(WorkArea, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations)

end



function FruitPreparerExtended.registerFunctions(vehicleType)

	SpecializationUtil.registerFunction(vehicleType, "onFieldDataUpdateFinished", FruitPreparerExtended.onFieldDataUpdateFinished)
	SpecializationUtil.registerFunction(vehicleType, "processFruitPreparerArea", FruitPreparerExtended.processFruitPreparerArea)

end



function FruitPreparerExtended.registerOverwrittenFunctions(vehicleType)

	SpecializationUtil.registerOverwrittenFunction(vehicleType, "loadWorkAreaFromXML",					FruitPreparerExtended.loadWorkAreaFromXML)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getDoGroundManipulation",				FruitPreparerExtended.getDoGroundManipulation)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "doCheckSpeedLimit",					FruitPreparerExtended.doCheckSpeedLimit)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAllowCutterAIFruitRequirements",	FruitPreparerExtended.getAllowCutterAIFruitRequirements)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getDirtMultiplier",					FruitPreparerExtended.getDirtMultiplier)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getWearMultiplier",					FruitPreparerExtended.getWearMultiplier)

end



function FruitPreparerExtended.registerEventListeners(vehicleType)

	SpecializationUtil.registerEventListener(vehicleType, "onLoad", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onReadUpdateStream", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onWriteUpdateStream", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onTurnedOn", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onTurnedOff", FruitPreparerExtended)
	SpecializationUtil.registerEventListener(vehicleType, "onStartWorkAreaProcessing", FruitPreparerExtended)

end



function FruitPreparerExtended:onLoad(savegame)

    local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.turnOnAnimation#name", "vehicle.turnOnVehicle.turnedAnimation#name") -- FS15 to FS17
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.turnOnAnimation#speed", "vehicle.turnOnVehicle.turnedAnimation#turnOnSpeedScale") -- FS15 to FS17
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.fruitPreparer#useReelStateToTurnOn") -- FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.fruitPreparer#onlyActiveWhenLowered") -- FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.vehicle.fruitPreparerSound", "vehicle.fruitPreparer.sounds.work") -- FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "vehicle.fruitPreparer.animationNodes.animationNode", "fruitPreparer") -- FS17 to FS19

    if self.isClient then
        spec.samples = {}
        spec.samples.work = g_soundManager:loadSampleFromXML(self.xmlFile, "vehicle.fruitPreparer.sounds", "work", self.baseDirectory, self.components, 0, AudioGroup.VEHICLE, self.i3dMappings, self)
        spec.animationNodes = g_animationManager:loadAnimations(self.xmlFile, "vehicle.fruitPreparer.animationNodes", self.components, self, self.i3dMappings)
    end

    spec.fruitType = FruitType.UNKNOWN

	spec.fruitTypes = {}
    local fruitTypesString = getXMLString(self.xmlFile, "vehicle.fruitPreparer#fruitType")
	local i = 0

	for substring in fruitTypesString:gmatch("%S+") do

		local fruitIndex = g_fruitTypeManager:getFruitTypeByName(substring)

		if fruitIndex ~= nil then
			spec.fruitTypes[i] = fruitIndex.index
			i = i + 1
		end

	end




    if #spec.fruitTypes > 0 then

		for _, index in pairs(spec.fruitTypes) do

			local desc = g_fruitTypeManager:getFruitTypeByIndex(index)

			if desc == nil then
				g_logManager:xmlWarning(self.configFileName, "Unable to find fruitTypeIndex '%d' in fruitPreparer", index)
			end

		end

    else
        g_logManager:xmlWarning(self.configFileName, "Missing fruitTypes in fruitPreparer")
    end

    spec.isWorking = false

end



function FruitPreparerExtended:onDelete()

	if self.isClient then

		local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

		g_soundManager:deleteSamples(spec.samples)
		g_animationManager:deleteAnimations(spec.animationNodes)

	end

end



function FruitPreparerExtended:onReadUpdateStream(streamId, timestamp, connection)

	if connection:getIsServer() then

		local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

		spec.isWorking = streamReadBool(streamId)

	end

end



function FruitPreparerExtended:onWriteUpdateStream(streamId, connection, dirtyMask)

	if not connection:getIsServer() then

		local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

		streamWriteBool(streamId, spec.isWorking)

	end

end



function FruitPreparerExtended:onTurnedOn()

	if self.isClient then

		local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

		g_soundManager:playSample(spec.samples.work)
		g_animationManager:startAnimations(spec.animationNodes)

	end

end



function FruitPreparerExtended:onTurnedOff()

	if self.isClient then

		local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

		g_soundManager:stopSamples(spec.samples)
		g_animationManager:stopAnimations(spec.animationNodes)

	end

end



function FruitPreparerExtended:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)

	local retValue = superFunc(self, workArea, xmlFile, key)

	if workArea.type == WorkAreaType.FRUITPREPARER then

		XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, key.."#dropStartIndex", key..".fruitPreparer#dropWorkAreaIndex") -- FS17 to FS19
		XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, key.."#dropWidthIndex", key..".fruitPreparer#dropWorkAreaIndex") -- FS17 to FS19
		XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, key.."#dropHeightIndex", key..".fruitPreparer#dropWorkAreaIndex") -- FS17 to FS19
		workArea.dropWorkAreaIndex = getXMLInt(xmlFile, key..".fruitPreparer#dropWorkAreaIndex")

	end

	return retValue

end



function FruitPreparerExtended:getDoGroundManipulation(superFunc)

	local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

	return superFunc(self) and spec.isWorking

end



function FruitPreparerExtended:doCheckSpeedLimit(superFunc)

	return superFunc(self) or (self:getIsTurnedOn() and (self.getIsImplementChainLowered == nil or self:getIsImplementChainLowered()))

end



function FruitPreparerExtended:getAllowCutterAIFruitRequirements(superFunc)

	return false

end



function FruitPreparerExtended:getDirtMultiplier(superFunc)

	local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

	if spec.isWorking then

		return superFunc(self) + self:getWorkDirtMultiplier() * self:getLastSpeed() / self.speedLimit

	end

	return superFunc(self)

end



function FruitPreparerExtended:getWearMultiplier(superFunc)

	local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

	if spec.isWorking then

		return superFunc(self) + self:getWorkWearMultiplier() * self:getLastSpeed() / self.speedLimit

	end

	return superFunc(self)

end



function FruitPreparerExtended.getDefaultSpeedLimit()

	return 15

end



function FruitPreparerExtended:onStartWorkAreaProcessing(dt)

	local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

	spec.isWorking = false

end



function FruitPreparerExtended:processFruitPreparerArea(workArea)

    local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]
    local workAreaSpec = self.spec_workArea
    local xs,_,zs = getWorldTranslation(workArea.start)
    local xw,_,zw = getWorldTranslation(workArea.width)
    local xh,_,zh = getWorldTranslation(workArea.height)
    local dxs, dzs = xs, zs
    local dxw, dzw = xw, zw
    local dxh, dzh = xh, zh

    if workArea.dropWorkAreaIndex ~= nil then

        local dropArea = workAreaSpec.workAreas[workArea.dropWorkAreaIndex]

        if dropArea ~= nil then

            dxs, _, dzs = getWorldTranslation(dropArea.start)
            dxw, _, dzw = getWorldTranslation(dropArea.width)
            dxh, _, dzh = getWorldTranslation(dropArea.height)

        end

    end

	if not self.requestedFieldData then

		self.requestedFieldData = true

		FSDensityMapUtil.getFieldStatusAsync(xs, zs, xw, zw, xh, zh, self.onFieldDataUpdateFinished, self)

	end

    local area = FSDensityMapUtil.updateFruitPreparerArea(spec.fruitType, xs,zs, xw,zw, xh,zh, dxs,dzs, dxw,dzw, dxh,dzh)

    if area > 0 then
        spec.isWorking = true
    end

    return 0, area

end



function FruitPreparerExtended:onFieldDataUpdateFinished(data)

	if not self.requestedFieldData then
		return
	end

    local spec = self["spec_" .. FruitPreparerExtended.modName .. ".FruitPreparerExtended"]

    local hasData = data ~= nil

    if hasData then

		for fruitDescIndex, _ in pairs(data.fruits) do

			if data.fruitPixels[fruitDescIndex] > 0 then

				for _, allowed in pairs(spec.fruitTypes) do

					if fruitDescIndex == allowed then

						spec.fruitType = fruitDescIndex

					end

				end

            end

        end

	end

	self.requestedFieldData = false

end